#!/usr/bin/perl -w

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules";
require "xhooks/lib/config.pl";

$cron_setting = "/Library/Preferences/Xhooks/edu.utah.scl.cron_config";

# Creates /etc/crontab based on a file that contains cron lines for each ip

my $cronfile = "";

if ( ! -f $cron_setting ) {
  print STDERR "The file $cron_setting does not exist!\n";
  exit 1;
}

wait_for_network();
my $my_ip = get_ip();

if ( $my_ip eq "" ) {
  print STDERR "I don't have an ip!\n";
  exit 1;
}

open ( FILEHANDLE, "< $cron_setting" );
my @lines = <FILEHANDLE>;
close ( FILEHANDLE );

foreach $current_line ( @lines ) {

  # check for comment or zero line
  if ( substr( $current_line, 0, 1 ) eq '#' || $current_line eq '' ) {
    next;
  }

  my @line_parts = split ( ',', $current_line );

  if ( compare_ips( $line_parts[0], $my_ip ) ) { 
    my $cronline = "$line_parts[1]";

    for ( $i=2;$i<=$#line_parts;$i++ ) {
      $cronline .= ",$line_parts[$i]";
    }

    $cronfile .= $cronline;

    print "Adding \"$cronline\".\n" if ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'});
  }

}

if ( $cronfile ) {

  open ( FILEHANDLE, "> /etc/crontab" );
  print FILEHANDLE $cronfile;
  close ( FILEHANDLE );

} else {
  print "Nothing found to add to /etc/crontab.\n" if ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'});
}

print "Created /etc/crontab.\n" if ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'});

exit 0;
