#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

#
# This script modifies or creates the file /Library/Preferences/com.apple.SystemLoginItems.plist
# It requires the array @SYSTEM_LOGIN_ITEMS be set either in the command file or entman_settings_global.pl.
#

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
use Foundation;
require "xhooks/lib/perlplist.pl";

if ( ! defined @SYSTEM_LOGIN_ITEMS ) {
  entman_logger ( "log drawer", "SYSTEM_LOGIN_ITEMS array was not defined." );
  exit 0;
}

if ( $#SYSTEM_LOGIN_ITEMS < 0 ) {
  entman_logger ( "log drawer", "There are no system login items to write." );
  exit 0;
}

my $system_login_items_file = "/Library/Preferences/com.apple.SystemLoginItems.plist";

entman_logger ( "log drawer", "Modding $system_login_items_file." );

%appHash=();
if ( -e $system_login_items_file ) {
  $dict = loadDefaults( $system_login_items_file );
  @appArray = perlArrayFromNSArray( getPlistObject( $dict, "AutoLaunchedApplicationDictionary" ), 1 );
  foreach $jj ( @appArray ) {
	$appHash{$$jj{"Path"}}=1;
  }
} else {
  $dict = NSMutableDictionary->dictionary();
  @appArray = ();
}

foreach $ii ( @SYSTEM_LOGIN_ITEMS ) {
  if (! $appHash{$ii}) {
	push @appArray, { "Path" => $ii };
  }
}

setPlistObject( $dict, "AutoLaunchedApplicationDictionary", cocoaArrayFromPerlArray( @appArray ) );
saveDefaults( $dict, $system_login_items_file );

system "/usr/bin/plutil", "-convert", "binary1", $system_login_items_file;

entman_logger ( "log drawer", "Wrote com.apple.SystemLoginItems.plist" );

exit 0;
