#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules";
require "xhooks/lib/config.pl";

my $computer_names = "/Library/Preferences/Xhooks/computer_names";

# This script sets the computer name.  You can set the name by matching MAC
# address or IP, or if there is no match, then use the DNS, or "no_network" if
# there is no network.  The MAC addy or IP names are stored in
# /etc/entman/conf/computer_names.  If the file doesn't exist or there is no
# match, then the first part of the DNS name is used.

my $seconds_to_wait_for_network = 60;

################################################################################
# Change computer name
#
my $computer_name = "";

if ( -e $computer_names ) {

  chomp (my $MACaddr = `ifconfig en0 | grep ether | cut -f2 -d' '` );

  wait_for_network();
  my $my_ip = get_ip();

  open ( FILEHANDLE, "< $computer_names" );
  while ( <FILEHANDLE> ) {
    my $current_line = $_;

    # check for comment or zero line
    if ( substr( $current_line, 0, 1 ) eq '#' || $current_line eq '' ) {
      next;
    }

    my @line_parts = split ( ',', $current_line );
    
    my $match = 0;

    if ( grep ( /:/, $line_parts[0]) ) {
      $match = 1 if ( lc ( $line_parts[0] ) eq lc ( $MACaddr ) );
    } else {
      $match = 1 if ( compare_ips( $line_parts[0], $my_ip ) );
    }
    
    if ( $match ) { 
      $computer_name = "$line_parts[1]";
      for ( $i=2;$i<=$#line_parts;$i++ ) {
        $computer_name .= ",$line_parts[$i]";
      }
      last;
    }

  }
  close ( FILEHANDLE );

}

if ( $computer_name eq "" ) {

  foreach ( 1..$seconds_to_wait_for_network ) {
    my $hostname = `hostname -s`;
    chomp $hostname;
    $computer_name = $hostname;
    if ( $hostname =~ /^local./ or $hostname =~ /^localhost/ ) {
      sleep 1;
    } else {
      last;
    }
    print "hi\n";
  }
  
}

if ( -e $radmind_error_file ) {
  $computer_name .= "_";
}

print "Setting computer name to $computer_name.\n" if ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'});

system "/usr/sbin/scutil --set ComputerName $computer_name";
system "/usr/sbin/scutil --set LocalHostName $computer_name";

exit 0;
