#!/usr/bin/perl

################################################################################
# Copyright (c) 2008 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

# This file REQUIRES that you set DESKTOP_IMAGE_LOCATION in the command file.
# This variable is the path of the desktop image.  In other words, when you
# go to System Prefs and set the desktop picture, whatever that path is,
# that is what the variable should be.  Set the variable in a command file.

# load config file (this loads the radmind_xhooks_conf file)
use lib "/Library/Xhooks/Modules";
require "xhooks/lib/config.pl";
require "edu.utah.scl.display/display_conf.pl";

if ( ! defined $DISPLAY_RESOLUTION ) {
  print STDERR "ERROR $0 - The variable DISPLAY_RESOLUTION was not defined.\n";
  exit 1;
}

if ( ! defined $DESKTOP_IMAGE_LOCATION ) {
  print STDERR "ERROR $0 - The variable DESKTOP_IMAGE_LOCATION was not defined.\n";
  exit 1;
}

# Get the image filename (basically the resolution)
my $image_file_name;
if ( $DISPLAY_RESOLUTION =~ /^(\d+)\s+(\d+)/ ) {
  $image_file_name = "$1_$2.jpg";
} else {
  print STDERR "ERROR $0 - Could not find get the image resolution.\n";
  exit 1;
}

my $dated_image_folders = "/Library/Preferences/Xhooks/edu.utah.scl.desktop/dated";
my $default_image_folder = "/Library/Preferences/Xhooks/edu.utah.scl.desktop/default";
my $image_file_path;

# See if there is a dated file
my $dated_folder = get_dated_folder_for_path( $dated_image_folders );

if ( -e "$dated_folder/$image_file_name" ) {

  # Use dated image
  $image_file_path = "$dated_folder/$image_file_name";

} elsif ( -e "$default_image_folder/$image_file_name" ) {

  # Use default image
  $image_file_path = "$default_image_folder/$image_file_name";

} else {

  print STDERR "ERROR $0 - Could not find $default_image_folder/$image_file_name.\n";
  exit 1;

}

print "cp $image_file_path $DESKTOP_IMAGE_LOCATION\n" if ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'});
system "cp", "$image_file_path", "$DESKTOP_IMAGE_LOCATION";

exit 0;
