#!/usr/bin/perl

################################################################################
# assimilateUser.pl
#
# The purpose of this script is to cleanup the user home folder (/Users/<name>),
# so that a "clean" template folder is available at the end of this script.
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

# NOTE: This script is very old and crude and up for a major overhaul...

################################################################################
# INSTRUCTIONS:
#
# When:
# This script is called by loginhook.pl at login.  It is also called by
# idlescript.pl on kiosks when the screensaver is activated.
#
# Requirements:
# 1. This script is dependent on create_cached_home_folders.pl, which creates
# the template home folders if none exist in the cache.  The cache home folders
# should be located at /etc/entman/system/home_folder/home_folder_cache.
# 2. This script is dependent on the file
# /etc/entman/conf/entman_config.pl (and that file has several other
# requirements).
# 3. This script requires the username of the user logging in passed in as the
# 1st parameter.  Ex. /Library/ULabMin/Private/scripts/assimilateUser.pl joe1234
#
# How the script works:
#
# Several values are pulled from the entman config files, notably the location 
# of the home folder (ex. /Users/guest).
#
# STEP 1 - Removes any /Users/guest folder that the current user expects
#
# STEP 2 - Creates/moves new home folder
#          If there are no home folders in the cache, it calls 
# create_cached_home_folders.pl (this process right here is kinda goofy--might 
# have bugs--and needs improvement, basically create_cached_home_folders.pl is a 
# goofy script...).  When there are home folders available, the a folder is 
# moved out of the cache to the active home folder location (ex. /Users/guest).  
# The process of choosing which home folder to move is basically: count the 
# number of items in the cache folder, move the folder with that name.  So if 
# there are 5 folders, then /etc/entman/system/home_folder/home_folder_cache/5 is 
# choosen.  This process could be improved as well.
#
# STEP 3 - Fix new home folder
#          The owner of the home folder is assigned to the user logging in.  The 
# file ~/.username is also created ($login_name_trigger).
#
# STEP 4 - Deal with lost and found
#          If the user who is currently logged in has a folder 
# in /etc/entman/system/home_folder/lost_and_found, then that folder is moved to the 
# Desktop.
#
# Cautions:
# If /etc/entman/system/home_folder/home_folder_cache/ contains anything other than 
# template home folders, kiss this whole process goodbye, it wont work.  The 
# create_cached_home_folders.pl script is actually suppose to see if there is a 
# problem and fix it (by deleting everything in that folder and starting 
# over...)
# Every now and then the home folder isn't assimilated, but a system home folder 
# is created.  I have no idea why, and it hardly ever happens, so I haven't been 
# able to figure out if it is a problem with something else and not even this 
# script.
# This script is archaic and does not clean up very well.  It is probably kinda 
# slow too (but not as slow as ditto'ing a whole home folder, so it is still 
# worth it.  We plan to fix the code in this script someday.
#
################################################################################

use Fcntl qw/O_WRONLY O_CREAT O_EXCL/;
use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
require "edu.utah.scl.home_folder/lib/home_folder_lib.pl";

$create_lost_and_found_script = "/Library/Xhooks/Modules/edu.utah.scl.home_folder/create_lost_and_found.pl";
$lostAndFoundReadmeName = " README.txt";
$global_lostAndFoundReadme = "$entman_folder/conf/home_folder/lostAndFoundReadme";
$homefolder_lostandfound = "Desktop/Lost & Found";
my $assimilateWaitTriggerFile = "/Library/Preferences/Xhooks/triggerfiles/movingHomeFolders";

$print_debug = 1;

################################################################################
# GO:
$rightnow = time;
entman_logger ( "log", "Started..." );

# Make sure I know the username.
if ( ! defined $ARGV[0] ) {
  entman_logger ( "log", "Quiting because I don't know the username (ARGV[0] was not defined)." );
  exit 1;  
}
$theuser = $ARGV[0];


##################################
assimilateUser(); # does the deed.
##################################

$totaltime = time - $rightnow;
entman_logger ( "log", "Done.  Took $totaltime." );

# QUIT:
say( "assimilateUser done" );
exit 0;


################################################################################
################################################################################
################################################################################


################################################################################
# The main worker bee
sub assimilateUser {
  say( "assimilateUser" );

  ################################################################################
  # STEP 1 - MAKE SURE OLD HOME FOLDER IS GONE
  if ( -e $userHomeFolder ) {  # $userHomeFolder ex. /Users/student
    entman_logger ( "log", "The folder, $userHomeFolder, exists." );
    system $create_lost_and_found_script;
    
    #
    # put the script in this script so more processes do not have to start, because the new script takes forever to launch.
    #
    
  }

#
# Maybe just move the folder out of the way *real quick* and deal with it later, after running chown (step 3)
# (in an attempt to keep the hot cache empty--probably useless...)
#


  ################################################################################
  # STEP 2 - CREATE/MOVE NEW HOME FOLDER

  my $use_new_folder = 1;
  if ( -e "$global_home_folder_lost_and_found/$theuser" ) {

    # loop through folders and look for ~/.dontcleanup and use it instead of cache
    my @oldFolders = getFileFolderList("$global_home_folder_lost_and_found/$theuser");
    foreach $folder ( @oldFolders ) {
      if ( -e "$global_home_folder_lost_and_found/$theuser/$folder/.dontcleanup" ) {

        say( "Move the saved home folder to be used" );
        entman_logger ( "log", "Using an old home folder: $global_home_folder_lost_and_found/$theuser/$folder" );
        print "/bin/mv \"$global_home_folder_lost_and_found/$theuser/$folder\" \"$userHomeFolder\"\n" if $print_debug;
        system "/bin/mv \"$global_home_folder_lost_and_found/$theuser/$folder\" \"$userHomeFolder\"";

        # If there is nothing in the lost and found, well, just delete the whole thing.
        if ( findNumberOfFoldersAndFiles( "$global_home_folder_lost_and_found/$theuser" ) < 1 ) {
          print "/bin/rm -rf \"$global_home_folder_lost_and_found/$theuser\"\n" if $print_debug;
          system "/bin/rm -rf \"$global_home_folder_lost_and_found/$theuser\"";
        }

        $use_new_folder = 0;
        last;

      }
    }
  }
  
  if ( $use_new_folder ) {

    # Wait, just in case home folders are being moved.
    my $wait = 1;
    my $counter = 1;
    while ( $wait and $counter < 100 ) {
      entman_logger ( "log", "Waiting for assimilate trigger file: $assimilateWaitTriggerFile...\n" );
      $wait = 0;
      $counter++;
      if ( open(FH, "<", $assimilateWaitTriggerFile) ) {
        entman_logger ( "log", "Trigger exists: $assimilateWaitTriggerFile\n" );
        $wait = 1;
        close(FH);
        sleep 1;
      } else {
        sysopen(FH, $assimilateWaitTriggerFile, O_WRONLY | O_CREAT | O_EXCL) or $wait = 2;
      }
      if ( $wait == 2 ) {
        entman_logger ( "log", "Race condition averted!\n" );
        sleep 1;
      }
    }
    if ( $counter > 99 ) {
        entman_logger ( "log", "Error, error, error!  No home folder cache!  Help me, I'm dying..." );
        exit 1;
    }

    # lock obtained, proceed

    # Make sure there are home folders
    if ( count_home_folder_cache_folders() ) {

      $newHomeFolder = "$homeFolderBackups/".count_home_folder_cache_folders();

      # Move the clean home folder to be used
      say( "Move the clean home folder to be used" );
      entman_logger ( "log", "Using a cached home folder: $newHomeFolder" );
      print "/bin/mv \"$newHomeFolder\" \"$userHomeFolder\"\n" if $print_debug;
      system "/bin/mv \"$newHomeFolder\" \"$userHomeFolder\"";

      close(FH);
      unlink $assimilateWaitTriggerFile;

    } else {

      close(FH);
      unlink $assimilateWaitTriggerFile;
  
      create_home_folder ( "$userHomeFolder" );
    
    }
    

  }

  ################################################################################
  # STEP 3 - FIX NEW HOME FOLDER

  # Change permissions
  say( "Change permissions" );
  print "/usr/sbin/chown -R $theuser \"$userHomeFolder\"\n" if $print_debug;
  system "/usr/sbin/chown -R $theuser \"$userHomeFolder\"";

  # Save the current username
  say( "Save the current username" );
  $createlabLoginName = "/bin/echo $theuser > \"$userHomeFolder/$login_name_trigger\"";
  system "$createlabLoginName\n" if $print_debug;
  system "$createlabLoginName";

  ####################################
  # STEP 4 - DEAL WITH LOST AND FOUND
  # If this doesn't work, it is probably because you are using special characters 
  # (other than " " or "&", which this script will fix).

  if ( -e "$global_home_folder_lost_and_found/$theuser" ) {
    # move lost and found to correct place.
    $new_lost_and_found = "$userHomeFolder/$homefolder_lostandfound";
    entman_logger ( "log", "/bin/mv \"$global_home_folder_lost_and_found/$theuser\" \"$new_lost_and_found\"\n" );
    print "/bin/mv \"$global_home_folder_lost_and_found/$theuser\" \"$new_lost_and_found\"\n" if $print_debug;
    system "/bin/mv \"$global_home_folder_lost_and_found/$theuser\" \"$new_lost_and_found\"";

    # Add Read me
    print "/bin/cp \"$global_lostAndFoundReadme\" \"$new_lost_and_found/$lostAndFoundReadmeName\"\n" if $print_debug;
    system "/bin/cp \"$global_lostAndFoundReadme\" \"$new_lost_and_found/$lostAndFoundReadmeName\"";
    print "/usr/sbin/chown $adminusername \"$new_lost_and_found/$lostAndFoundReadmeName\"\n" if $print_debug;
    system "/usr/sbin/chown $adminusername \"$new_lost_and_found/$lostAndFoundReadmeName\"";
    print "/usr/bin/chgrp $enduser_group \"$new_lost_and_found/$lostAndFoundReadmeName\"\n" if $print_debug;
    system "/usr/bin/chgrp $enduser_group \"$new_lost_and_found/$lostAndFoundReadmeName\"";
    print "/bin/chmod 640 \"$new_lost_and_found/$lostAndFoundReadmeName\"\n" if $print_debug;
    system "/bin/chmod 640 \"$new_lost_and_found/$lostAndFoundReadmeName\"";
  }

}
