#!/usr/bin/perl

# (don't do -w because right now the script is launched by cron... and we don't want noise that will be emailed to root)

################################################################################
# idlescript.pl
#
# v 1.2.7c
#
# This script can be configured many ways, depending on what "trigger files"
# are present on the hard disk (in other words, I am a global script, and the
# files on my hard disk tells me what to do/be).
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
# v 1.2.7c 2006.10.02 I'm probably forgetting a lot of changes... Anyway, added
#                     a conf file at /etc/entman/idle_conf.pl
# v 1.2.7b 2005.02.23 Removed Safari version checking (shouldn't be in here)
# v 1.2.7a 2005.01.25 Fixed status reporting stuff so it works right.
# v 1.2.7 2005.01.12 Added status reporting for logged out status.
# v 1.2.6 2004.10.06 Added lookups for ica files (for internet kiosks so they can open Citrix files)
# v 1.2.5 2004-07/08 Added lots of stuff and made it part of ulabmin .7.
#          Added new_presentation script to this file
# v 1.2.4 2004-03-06 Small stuff.  Like logging to figure out why it doesn't always open a new
#          browser window.
# v 1.2.3 2004-02-02 Added check for Safari 1.2.
#          Added non-running code back in since a crashed browser may not be
#          relaunched by the system.
# v 1.2.2 2004-01-30 Fixed the way the script found if Safari or IE was the default browser.
# v 1.2.1 2004-01-28 Added auto detect for Safari
#
# v 1.2 Changes since 1.1.1
# 04-01-23 Added support for Safari (only tested briefly)
# 04-01-23 Added more ARD trigger files
# 04-01-23 Rewrote the main loop so it makes sense
# 03-12-29 Tried to add pasteboard cleanup, but it didn't work...
#
# v 1.1.1 major changes
# ??-??-?? Rewrote the main loop trying to make it make sense
# ??-??-?? Added some code that detected if it was 10.3 or not (and fork accordingly)
# ??-??-?? Added support for ipopfeed, but didn't finish
# ??-??-?? Centeralized all variables into ulabmin_do_not_modify_config.pl
#
################################################################################

# To create a kiosk, you need to run this command:
# su kiosk
# defaults write com.apple.loginwindow Finder /Applications/Safari.app
# (or whatever the path is to your "Finder replacement", Safari or Internet Explorer...
# and then copy this file to all of our computers in the kiosk user's home folder
# ~/Library/Preferences/com.apple.loginwindow.plist


use lib "/Library/Xhooks/Modules";
require "xhooks/lib/config.pl";

if ( -e "/Library/Xhooks/Modules/edu.utah.scl.display/display_conf.pl" ) {
  require "edu.utah.scl.display/display_conf.pl";
}

my $availablity_lib = "/Library/Xhooks/Modules/edu.utah.scl.availability/availability_lib.pl";
if ( -e $availablity_lib ) {
  require "edu.utah.scl.availability/availability_lib.pl";
}

$logoutNowScript = "/Library/Xhooks/Modules/idle/LogoutNow.app";
$new_presentation_script = "$entman_folder/lib/idle/newPresentation.pl";
$pathToScreenPreserver = "/usr/local/bin/Screen Preserver.app/Contents/MacOS/Screen Preserver";
$kiosk_cleanup_triggerfile = "/tmp/cleanup_kiosk_now";
$loggingOutTriggerFile = "/tmp/logoutNow"; # if exists, pause idlescript and wait until loginwindow is running by root
$run_startup_post_maintenance = "/Library/Preferences/Xhooks/triggerfiles/run_startup_post_maintenance";
$radmind_finished = "/Library/Preferences/Xhooks/triggerfiles/radmind_finished";
$login_hook_running = "/Library/Preferences/Xhooks/triggerfiles/login_hook_running";
$logout_hook_running = "/Library/Preferences/Xhooks/triggerfiles/logout_hook_running";

$idlescript_clean_up_allowed = 1;
$openCitrix_ica_files_on_kiosks = 1; # Set this to 1 if you want kiosks to auto open
									 # Citrix files when they are downloaded.
my $wait_for_radmind = "/var/log/wait_for_radmind";

if ( ! defined $DISPLAY_RESOLUTION or $DISPLAY_RESOLUTION eq "" ) {
  $idlescript_kiosk_safari_window_size = "{0, 22, 1024, 764}";
} else {
  if ( $DISPLAY_RESOLUTION =~ /^(\d+)\s+(\d+)/ ) {
    $idlescript_kiosk_safari_window_size = "{0, 22, $1, $2}";
  } else {
    $idlescript_kiosk_safari_window_size = "{0, 22, 1024, 764}";
  }
}

################################################################################
# loginwindow.app notes
#
# This is the loginwindow behavior in 10.2.2.  I hope Apple doesn't change it in
# the future.
#
# Loginwindow first launches with a pid and is run by root.  When it logs in, it
# runs to the user who logged in, but has the same pid.  When the user logs out,
# for a moment, it reverts back to root with the same pid.  But then it
# relaunches with a new pid and is root again.
#
# Like this (pid, user, process):
#
#    LOGGED OUT
# 17549 root /System/Library/CoreServices/loginwindow.app/Contents/MacOS/loginwindow
#    LOGGED IN
# 17549 guest /System/Library/CoreServices/loginwindow.app/Contents/MacOS/loginwindow
#    LOGGING OUT
# 17549 guest /System/Library/CoreServices/loginwindow.app/Contents/MacOS/loginwindow
# 17549 root /System/Library/CoreServices/loginwindow.app/Contents/MacOS/loginwindow
#    LOGGED OUT
# 20006 root /System/Library/CoreServices/loginwindow.app/Contents/MacOS/loginwindow
#
################################################################################

################################################################################
# Other notes:
#
# `who | grep console | awk \'{ print \$1 }\'` returns the username or "" if
# logged out.  It is the quickest way to get username. Except the loginhook runs
# before `who | grep console` returns anything, so if you touch a file in the
# loginhook, then that is quicker than `who | grep console`
#
# The only publicly known way (before 10.3) was to determine idle time is by
# configuring the screen saver to activate after a given time and checking to
# see if it is running.  There are several ways to do this now.  But this script
# has not been fully updated to use those.
#
################################################################################

#################################################################################
# Change these
#

$timeTillNextCheck = 3; # wait this many seconds before iterating the main event loop
$timeToDelayCleanup = 10; # give the user this many seconds to deactive screen saver and cancel cleanup
$screen_saver_time_to_activate = 5; # Minutes before screen preserver activates (over login panel only)

$use_screen_preserver = 1; # The system screensaver can't be configured in 10.4.  You can run it, but you can't give it any prefs, like pictures folder for ImageShow :'(
                           # To use the system screensaver, just set this to 0 (and make sure you have prefs configured for the module
                           
$system_screensaver_module = "ImageShow";

#############################################
# Debug

$debug_log = "/tmp/idlescript_debug";
unlink $debug_log if ( -e $debug_log );

#############################################
# Set the default app!
#
if ( $this_computer_is_a_kiosk_computer ) {
  # What's your flavor?
  $mainAppSystemname = "Safari";
  $mainAppFilename = "Safari";
} else {
  $mainAppSystemname = "Finder"; # this is the name as it appears to AppleEvents
  $mainAppFilename = "Finder"; # this is the name as it appears to launch events, whatever...
}

#############################################
# Finish initializing
#
$loginwindow = "/System/Library/CoreServices/loginwindow.app/Contents/MacOS/loginwindow";

my $my_ip = get_ip();
my $my_decimal_ip = decimial_ip ($my_ip) % (60*5);

$debug = 0;
$debugWithoutVoice = 0;

#############################################
# RUN
#

init(); # Initialize stuff that changes again while running

idle_loop(); # Start a loop that never ends.

exit 0; # This should never be reached.

################################################################################
# Idle loop
################################################################################

################################################################################
sub idle_loop {
  while ( 1 ) {

    say( "running" );
    #make sure changes while I'm sleeping don't mess me up!
    if ( $sessionID != getPID( $loginwindow ) ) {
      init();
    }

    ######################################
    ######################################
    ## CHECK FOR THINGS THAT STOP NORMAL OPPERATING PROCEDURES...
    ######################################
    ######################################

    # check for radmind (this check should check for iHook too.
    # Actually, the radmind scripts should create a trigger file...)
#    if ( isThisAppRunning( "iHook", "root" ) or
#         isThisAppRunning( "fsdiff", "root" ) or
#         isThisAppRunning( "ktcheck", "root" ) or
#         isThisAppRunning( "lapply", "root" ) )
    if ( ! -e $radmind_finished or -e $run_startup_post_maintenance ) {
      say( "running radmind or post maintenance" );
      if ( $this_computer_is_a_kiosk_computer and isThisAppRunning( "$mainAppSystemname", "$kioskusername" ) ) {
        system "killall $mainAppSystemname";
      }
      sleep $timeTillNextCheck;
      next;
    }

    # check to make sure the Mac isn't logging out because it will screw with everything if it is.
    if ( -e $loggingOutTriggerFile ) {
      say( "logging out" );
      my $time_to_logout = 15;
      if ( $logout_timer == 0 ) {
        say( "will force logout in $time_to_logout seconds" );
        # display message that I'm going to logout!
        $logout_timer = time;
      }
      if ( time - $logout_timer > $time_to_logout ) {
        if ( ! -e $logout_hook_running ) {
          # log me out.
          say( "force logout now" );
          system "\"$killsumapps_script\" ". getUsername();
          # maybe kick off an applescript that logs the user out and wait to let the logout hook run.
          system "/usr/bin/killall loginwindow";
          # or just run it here anyway and pop up ihook to prevent login
        }
      }
      sleep 1;
      next;
    }


    # What could happen before the quit trigger file is found?
    # * $killsumapps_script
    # * /usr/bin/killall loginwindow


    # check for quit file
    if ( -e $wait_for_radmind ) {
      say( "quit idle script" );
      sleep $timeTillNextCheck;
      next;
    }


    # What does this script stop for (in order)
    # * running fsdiff, ktcheck, or lapply
    # * logging out
    # * quit trigger file

    ######################################
    ######################################
    ## NORMAL OPERATING PROCEEDURES FOLLOW
    ######################################
    ######################################

    $idlescript_username = getUsername();
    if ( $idlescript_username eq $adminusername ) {

      ########################
      # Admin is logged in
      $exception = 1;
      say( "hello master" );
      sleep $timeTillNextCheck;
      next;

    } elsif ( $idlescript_username ne "" ) {

      ########################
      # Logged in
      if ( $this_computer_is_a_kiosk_computer ) {

        ########################
        # KIOSK
        if ( -e $kiosk_cleanup_triggerfile ) {
          kioskCleanup();
          unlink $kiosk_cleanup_triggerfile;
          next;
        }
        if ( $this_computer_is_authenticated ) {
          authenticated_loggedin_stuff();
          kiosk_authenticated_stuff();
        } else {
          kiosk_autologin_stuff();
        }
      } elsif ( $this_computer_is_a_presentation_computer ) {
        if ( ! defined $presentationTimer ) {
          $presentationTimer = 0;
        } elsif ( $presentationTimer >= 5 ) {
          system "\"$new_presentation_script\"";
          $presentationTimer = 0;
        } else {
          $presentationTimer++;
        }
      } else {

        ########################
        # LAB
        if ( $this_computer_is_authenticated ) {
          authenticated_loggedin_stuff();
          labStuff();
        } else {
          #labStuff(); (not tested, but should work in theory)
        }

      }

    } else {

      ########################
      # Logged out
      if ( $this_computer_is_authenticated ) {
        authenticated_loggedout_stuff();
      } else {
        authenticated_loggedout_stuff();
        if ( $this_computer_is_a_kiosk_computer ) {
          # shouldn't happen unless admin is logging in.
          # Since this is suppose to be autologin, it might help to add code here that reboots to login...
        } elsif ( $this_computer_is_a_presentation_computer ) {
          # shouldn't happen unless admin is logging in.
          # Since this is suppose to be autologin, it might help to add code here that reboots to login...
        }
      }
      
    }
    sleep $timeTillNextCheck;
  }
}

################################################################################
# Branches of the idle loop
################################################################################

sub kiosk_authenticated_stuff {
  if ( isKioskIdle() ) {
    say( "idle" );
    initiate_logout();
  } else {
    kioskStuffInUse();
  }
}

sub kiosk_autologin_stuff {
  if ( isKioskIdle() ) {
    kioskStuffIdle();
  } else {
    kioskStuffInUse();
  }
}

################################################################################
sub kioskStuffIdle {
  say( "idle" );
  # Check to see if clean up needs to be done
  if ( $needsCleanUp ) {
    say( "needs clean up" );
    $rightNow = time;
    if ( $timeSinceLastCleanUp == 0 ) {
      $timeSinceLastCleanUp = $rightNow;
    } elsif ( ( ( $rightNow ) - $timeSinceLastCleanUp ) > $timeToDelayCleanup ) {
      if ( $idlescript_clean_up_allowed ) {
        kioskCleanup();
      }
      $needsCleanUp = 0;
      $timeSinceLastCleanUp = 0;
    }
  } else {
    home_folder_management();

    empty_trash(); # make this only happen if home_folder_managerment didn't run this loop

  }
}

sub empty_trash {
  if ( $trashNeedsCleaning ) {
    system "/usr/bin/chflags -R nouchg \"$trash_can/\"* ; /bin/rm -rf \"$trash_can/\"* &";
    $trashNeedsCleaning = 0;
  }
}

################################################################################
sub kioskStuffInUse {
  say( "not idle" );
  $needsCleanUp = 1;
  $trashNeedsCleaning = 1;
  $needsMoreHomeFolders = 1;

  #  ------------------------------------------------------------
  # Mac is not idle
  # Make sure that the main app is launched (don't need to do this in Panther)

  if ( ! isThisAppRunning( $mainAppFilename, $idlescript_username ) ) {
    say( "$mainAppFilename not running by user $idlescript_username" );
    $rightNow = time;
    if ( $timeSinceNoWindow == 0 ) {
      $timeSinceNoWindow = $rightNow;
      say( "wait a little more" );
    } elsif ( ( ( $rightNow ) - $timeSinceNoWindow ) > $timeToDelayWindowReset ) {
      #  ------------------------------------------------------------
      # MAIN APP IS NOT RUNNING!  Launch it!
      launchMainKioskApp();
      $timeSinceNoWindow = 0;
    } else {
      say( "wait a little more" );
    }
  } else {
    # check to make sure a window is open
    say( "checking browser window" );
    makeSureBrowserWindowIsOpen();
    # Not sure if the following is needed:
    ## Remove iPopfeed
    #end_ipopfeed( "kiosk_launchbrowser" );
  }

  # Open Citrix file if present...
  if ( $openCitrix_ica_files_on_kiosks ) {
    @number_of_files = `/bin/ls -1 \"$userHomeFolder/Desktop/\"*\".ica\" 2> /dev/null`;
    foreach $file ( @number_of_files ) {
      chomp $file;
      system "open \"$file\"";
      sleep 5;
      unlink $file;
    }
  }
}

################################################################################
sub labStuff {
  # Do here what you want.  We don't do anything...

  #if ( isThisAppRunning( "ScreenSaverEngine", $idlescript_username ) ) {
  #  say( "idle" );
  #  if ( $this_computer_is_authenticated ) {
  #    initiate_logout();
  #  }
  #} else {
  #  say( "not idle" );
  #}
}

################################################################################
sub authenticated_loggedin_stuff {
  $needsMoreHomeFolders = 1;
  $trashNeedsCleaning = 1;
  ########################
  # Kill screen preserver if need be
  if ( $use_screen_preserver ) {
    if ( isThisAppRunning( $pathToScreenPreserver, "root" ) ) {
      $screensaver = `ps -wwxo pid,command -U root | grep \"$pathToScreenPreserver\" | grep -v grep`;
      $_=substr( $screensaver,0,6 );
      $bla=/\s*(\d+)\s*/;
      system "/bin/kill $1";
    }
  }
}

################################################################################
sub authenticated_loggedout_stuff {
  # make sure loginwindow is running (don't want to do anything while starting up).
  if ( isThisAppRunning( $loginwindow, "root" ) ) {
    my $idleTime = `/usr/sbin/ioreg -c IOHIDSystem | /usr/bin/perl -ane 'if (/Idle/) {\$idle=int((pop \@F)/1000000000); print \$idle,"\n"; last}'`;
    chomp $idleTime;
    if ( $idleTime > 240 ) {
      screen_saver();
      home_folder_management();
      empty_trash(); # make this only happen if home_folder_managerment didn't run this loop
    }
  }
  ################################################################################
  # Report that this computer is not available
  if ( defined $AVAILABILITY_URL and $AVAILABILITY_URL ne "" and -e $availablity_lib and ! $i_reported_my_status ) {
      report_availability ( "available", "" );
      $i_reported_my_status = 1;
  }
}


################################################################################
# Everything else
################################################################################

################################################################################
sub init {
  say( "New session" );
  $sessionID = getPID( $loginwindow );
  $logout_timer = 0;
  $needsMoreHomeFolders = 1;
  $trashNeedsCleaning = 1;
  $i_reported_my_status = 0;
  if ( $this_computer_is_a_kiosk_computer ) {
    $needsCleanUp = 1;
    $timeSinceLastCleanUp = 0;
    $timeSinceNoWindow = 0;
  }
  unlink $loggingOutTriggerFile;
}

################################################################################
sub home_folder_management {
  if ( $idlescript_clean_up_allowed and ! -e $login_hook_running ) {
    # Ideally make this detached process (I believe it isn't currently)
    if ( $needsMoreHomeFolders ) {
      $currentHomeFolders = count_home_folder_cache_folders();
      if ( $currentHomeFolders < 10 ) {
        say( "creating home folder" );
        $currentHomeFoldersPlus1 = $currentHomeFolders + 1;
        system "\"$create_cached_home_folders_script\" $currentHomeFoldersPlus1";
      } else {
        $needsMoreHomeFolders = 0;
      }
    }
  }
}

## AUTHENTICATION FUNCTIONS

################################################################################
sub screen_saver {
  $screensaver = $pathToScreenPreserver;
  $command = "\"$pathToScreenPreserver\" &";
  if ( ! isThisAppRunning( $screensaver, "root" ) ) {
    if ( ! isThisAppRunning( $pathToiHook, "root" ) ) {
      my $idleTime = `/usr/sbin/ioreg -c IOHIDSystem | /usr/bin/perl -ane 'if (/Idle/) {\$idle=int((pop \@F)/1000000000); print \$idle,"\n"; last}'`;
      chomp $idleTime;
      if ( ( $screen_saver_time_to_activate * 60 ) < $idleTime ) {
        system $command;
      }
    }
  }
}

sub initiate_logout {
  # log me out.
  say( "starting log out" );
  system "\"$killsumapps_script\" " . getUsername();
  system "/usr/bin/touch \"$loggingOutTriggerFile\"";
  system "/usr/bin/open \"$logoutNowScript\"";
}

## KIOSK FUNCTIONS

################################################################################
sub kioskCleanup {
  say( "cleaning up" );
#  system "/bin/echo -n \"\" | /usr/bin/pbcopy"; # clear the clipboard
  system "\"$assimilate_user_script\" $idlescript_username";
  sleep 3;
  system "\"$killsumapps_script\" " . getUsername();
  launchMainKioskApp();
}

################################################################################
sub launchMainKioskApp {
  say( "launching main application" );

  if ( isThisAppRunning( "iHook", "root" ) or
       isThisAppRunning( "fsdiff", "root" ) or
       isThisAppRunning( "ktcheck", "root" ) or
       isThisAppRunning( "lapply", "root" ) )
  { return; }

  if ( $mainAppSystemname eq "Safari" ) {
    doAppleEvent( "tell application \"$mainAppFilename\" to activate" ) ;
  } elsif ( $mainAppSystemname eq "Internet Explorer" ) {
    $numberOfWindows = doAppleEvent("tell application \"$mainAppFilename\" to Activate" );
    for ( $numberOfWindows = $numberOfWindows; $numberOfWindows > 1; $numberOfWindows-- ) {
      doAppleEvent( "tell application \"$mainAppFilename\" to CloseWindow every window" );
      sleep 1;
    }
  }
  makeBrowserWindowBig();
}

################################################################################
sub makeBrowserWindowBig {
  say( "launching main application" );
  if ( $mainAppSystemname eq "Safari" ) {
    doAppleEvent( "tell application \"$mainAppFilename\" to set bounds of window 1 to $idlescript_kiosk_safari_window_size" );
  } elsif ( $mainAppSystemname eq "Internet Explorer" ) {
    $makewindowbig = "tell application \"$mainAppSystemname\" to OpenURL ";
    $makewindowbig .= "\"javascript:window.moveTo(0,0);window.resizeTo((screen.width/1),(screen.height/1)); \"";
    doAppleEvent( $makewindowbig );
  }
}

################################################################################
sub makeSureBrowserWindowIsOpen {
  if ( getUsername() ne $idlescript_username ) {
    say( "wrong user" );
#    system "echo wrong user >> $debug_log";
    return 1;
  }
  if ( ! isThisAppRunning( $mainAppFilename, $idlescript_username ) ) {
    say( "app was suppose to be running, but isnt..." );
#    system "echo app was suppose to be running, but isnt... >> $debug_log";
    return 1;
  }

  if ( $mainAppSystemname eq "Safari" ) {
    my $result = doAppleEvent( "tell application \"$mainAppSystemname\" to count windows" );
    if ( $result eq "Bad" ) {
#      system "echo bad apple event >> $debug_log";
      return 1;
    }

    my $make_new_window = 0;
    if ( $result == 0 ) {
      $make_new_window = 1;
    } elsif ( $result == 1 ) {
      $url_of_window = doAppleEvent( "tell application \"$mainAppSystemname\" to set x to URL of document 1" );
      if ( $url_of_window eq "" ) {
        $make_new_window = 1;
      }
    }
    if ( $make_new_window ) {
#      system "echo 0 windows are open >> $debug_log";
      $debugresult = doAppleEvent( "tell application \"$mainAppSystemname\" to make new document" );
#      system "echo make new document result: $debugresult >> $debug_log";
      makeBrowserWindowBig();
    }

  } elsif ( $mainAppSystemname eq "Internet Explorer" ) {
    # This code is very old... probably should still work since IE isn't being updated.
    $numberOfWindows = doAppleEvent( "tell application \"$mainAppSystemname\" to set numOfWins to ListWindows" );
    if ( $numberOfWindows eq "\n" ) {
      say( "no browser window" );
      $rightNow = time;
      if ( $timeSinceNoWindow == 0 ) {
        $timeSinceNoWindow = $rightNow;
        say( "wait a little more" );
      } elsif ( ( $rightNow - $timeSinceNoWindow ) > $timeToDelayWindowReset ) {
        doAppleEvent( "tell application \"$mainAppSystemname\" to quit" );
        sleep 2;
        launchMainKioskApp();
        $timeSinceNoWindow = 0;
      } else {
        say( "wait a little more" );
      }
    } else {
      say( "window eye dees are $numberOfWindows" );
    }
  }
}

sub isKioskIdle {
  return isThisAppRunning( "ScreenSaverEngine", $idlescript_username );
}

## EVERYTHING ELSE

sub decimial_ip {
  my ( $the_ip) = @_;
  my @ip_parts = split '\.', $the_ip;
  my $total = shift @ip_parts;
  foreach my $ip_part ( @ip_parts ) {
    $total = $total* 256 + $ip_part;
  }
  return $total;
}

################################################################################
sub doAppleEvent {
  my ( $command ) = @_;

  # only do apple events if the loginwindow is running by the same user who started the main loop iteration
  if ( isThisAppRunning( $loginwindow, $idlescript_username ) ) {
    #print "osascript -e \'$command\'";
    $theResult = `osascript -e 'try' -e \'$command\' -e 'end try' `;
    #system "/usr/bin/osascript -e \'$command\'";
    print $theResult;
    return $theResult;
  } else {
    say( "can't execute applescript, not logged in" );
#    system "echo $loginwindow >> $debug_log";
#    system "echo $idlescript_username >> $debug_log";
    return "Bad"
  }
}
