#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

# How do you tell if there are any scheduled processes on a box.... 
# (If you aren't using cron)
#
# sudo launchctl list
#
# if in the list you have "edu.utah.scl.nightly.plist", then inspect the file:
#
# /Library/LaunchDaemons/edu.utah.scl.nightly.plist
#
# To see what the schedule is.
#
#	<key>ProgramArguments</key>
#	<array>
#		<string>/Library/Xhooks/Modules/edu.utah.scl.nightly/nightly.pl</string>
#		<string>MH</string>
#		<string>TWFSU</string>
#	</array>
#
# The first string after "/Library/Xhooks/Modules/edu.utah.scl.nightly/nightly.pl" is the days the machine
# will reboot and run radmind (in this case, Monday and Thursday).  The second string
# lists only the days that the machine will reboot.  To configure a machine with these
# settings, edit "/Library/Preferences/Xhooks/edu.utah.scl.nightly/radmind_reboot".  That file will tell you
# the format.
#
# To stop the schedule, type:
#
# sudo launchctl unload /Library/LaunchDaemons/edu.utah.scl.nightly.plist
#
# To restart it, type:
#
# sudo launchctl load /Library/LaunchDaemons/edu.utah.scl.nightly.plist

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

# This script should create a radmind and reboot launchdaemon plist.

$radmind_reboot = "/Library/Preferences/Xhooks/edu.utah.scl.nightly/radmind_reboot";
$nightly_script = "/Library/Xhooks/Modules/edu.utah.scl.nightly/nightly.pl";

use Foundation;
require "xhooks/lib/perlplist.pl";

if ( ! -f $radmind_reboot ) {
  entman_logger ( "log drawer", "The file $radmind_reboot does not exist." );
  exit 1;
}

wait_for_network();
my $my_ip = get_ip();

if ( $my_ip eq "" ) {
  entman_logger ( "log drawer", "I don't have an ip!" );
  exit 1;
}

open ( FILEHANDLE, "< $radmind_reboot" );
my @lines = <FILEHANDLE>;
close ( FILEHANDLE );

foreach my $current_line ( @lines ) {
 
  chomp $current_line;
  $current_line =~ s'  *' 'g;
  
  # check for comment or zero line
  if ( substr( $current_line, 0, 1 ) eq '#' or $current_line eq '' or $current_line eq ' ' ) {
    next;
  }

  my @line_parts = split ( ',', $current_line );
  if ( compare_ips( $line_parts[0], $my_ip ) ) { 

    my @args = split ( ' ', $line_parts[1] );

    if ( $#args == 3 ) { # 4 args
      create_radmind_ld_plist("edu.utah.scl.nightly", $args[0], $args[1], $args[2], $args[3]);
      last;

    } elsif ( $#args == 5 ) { # 6 args
      create_radmind_ld_plist("edu.utah.scl.nightly", $args[0], $args[1], $args[2], "-");
      create_radmind_ld_plist("edu.utah.scl.nightly2", $args[3], $args[4], "-", $args[5]);
      last;

    } else {
      entman_logger ( "log drawer", "Error: Wrong number of args! @args" );    
    }
    
  }

}

entman_logger ( "log drawer", "Done." );

exit 0;

sub create_radmind_ld_plist {
  my ( $identifier, $minute, $hour, $radmind_days, $reboot_days) = ( @_ );

  my %times = ( 
    Minute	=> cocoaInt( $minute ),
    Hour	=> cocoaInt( $hour )
  );

  my @progArgs = (
    $nightly_script,
    $radmind_days,
    $reboot_days
  );

  my %dict = (
    ProgramArguments		=> \@progArgs,
    Label					=> $identifier,
    StartCalendarInterval	=> \%times
  );

  my $save_file="/Library/LaunchDaemons/$identifier.plist";

  saveDefaults( cocoaDictFromPerlHash( %dict ), $save_file );
  entman_logger ( "log drawer", "Saved \"$hour:$minute $radmind_days, $reboot_days\" - to /Library/LaunchDaemons/$identifier.plist" );

}
