#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
use lib "/Library/Preferences/Xhooks/";
require "replicate/replicate_conf.pl";

################################################################################
# entman - modules/configure/PMD_replicate.pl
#
# This script creates a backup of files and folders that need to be replaced at
# every login.  Namely, cups settings, and files and folders that must be world
# world writable.  See LID_replicate.pl for the half performs the copies at 
# login.
#
################################################################################

$backup_cups_folder = "/Library/Preferences/Xhooks/replicate";

################################################################################
# Backup files that must be world writable
#
if ( $postm_backup_world_writable_switch ) {

  entman_logger ( "log drawer", "Replicating files to be replaced at login." );

  # Replicate world writable files (thx PSU for the name ;)
  foreach $item_to_replicate ( @login_replicate_items ) {

    if ( -e $item_to_replicate ) {

      # remove old if there... (shouldn't be)
      if ( -e "$writable_caches_folder/$item_to_replicate" ) {
        entman_logger ( "log drawer", "$writable_caches_folder/$item_to_replicate already existed.. deleting." );
        system "/usr/bin/chflags -R nouchg \"$writable_caches_folder/$item_to_replicate\"";
        system "/bin/rm -rf \"$writable_caches_folder/$item_to_replicate\"";
      }

      system "/usr/bin/ditto -rsrc \"$item_to_replicate\" \"$writable_caches_folder/$item_to_replicate\"";

      entman_logger ( "log drawer", "Backed up $item_to_replicate." );

    } else {
      entman_logger ( "log drawer", "$item_to_replicate does not exist." );
    }

  }

} else {

  entman_logger ( "log drawer", "Backing world writable files disabled." );

}

################################################################################
# Backup cups printer setting files (this is the same as replication, except
#                                    cups has to be restarted at login)
#
if ( $postm_login_manage_cups_settings_switch ) {
  
  entman_logger ( "log drawer", "Backing up cups settings." );

  # remove old if there... (shouldn't be)
  if ( -e "$backup_cups_folder" ) {
    entman_logger ( "log drawer", "$backup_cups_folder already existed.. deleting." );
    system "/bin/rm -fr \"$backup_cups_folder\"";
  }

  system "/usr/bin/ditto /etc/cups/ppd \"$backup_cups_folder/ppd\"";
  system "/usr/bin/ditto /etc/cups/printers.conf \"$backup_cups_folder/printers.conf\"";

  entman_logger ( "log drawer", "Backed up cups settings." );

} else {
  entman_logger ( "log drawer", "Backing up cups settings disabled" );
}

entman_logger ( "log drawer", "Done replicating items." );

exit 0;
