#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

my $ofpw_set = "$entman_folder/conf/secure/ofpw_set";

################################################################################
# entman - modules/configure/PMD_firmware_password.pl
#
# Sets the OFPW for this machine.
#
# To get the security password, you have to set it on a computer (using Apple's
# Open Firmware # Password utility) and then type "sudo nvram security-password"
# in the terminal.  Paste the result into the file /etc/entman/conf/ofpw_set.
#
# Example for /etc/entman/conf/secure/ofpw_set
#
# # PUT YOUR ENCRYPTED PASSWORD HERE:
# $security_password = "%00%00%00%00%00%00%00%00";
# 1;
#
#################################################################################

# Get the password

if ( ! -e $ofpw_set ) {
  exit 0;
}

require $ofpw_set;

if ( ! defined $security_password ) {
  entman_logger ( "log drawer", "$ofpw_set exists, but \$security_password is not defined." );
  exit 1;
}

my $platform = `/usr/sbin/system_profiler SPHardwareDataType | /usr/bin/grep "CPU Type" | /usr/bin/awk '{ print \$3 }'`;
chomp $platform;

if ( $platform eq "PowerPC" ) {

  # Set the password to a temp value
  # (In Tiger this is not suppose to be needed anymore...)
  # It definitely isn't needed on Intel boxen

  if ( ! -e "$entman_folder/bin/OFPW" ) {
    entman_logger ( "log drawer", "The OFPW tool ($entman_folder/bin/OFPW) does not exist." );
    exit 1;
  }

  system "$entman_folder/bin/OFPW", "-pass", "bla"; # don't change
  system "$entman_folder/bin/OFPW", "-mode", "1";  

} else {

  system "/usr/sbin/nvram", "security-mode=command";

}

# Now change the password for real

system "/usr/sbin/nvram", "security-password=$security_password";

# Delete password file

system "/usr/bin/srm", "$ofpw_set";

entman_logger ( "log drawer", "Done setting OFPW." );

exit 0;
