#!/usr/bin/perl -w

#
# Copyright (c) 2007 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#

use lib "/Library/Xhooks/Modules";
require "xhooks/lib/common.pl";

use Foundation;
require "xhooks/lib/perlplist.pl";

my $AddTextToImage = "/usr/local/bin/AddTextToImage";
my $isightcapture = "/usr/local/bin/isightcapture";
my $prefFile = "/Library/Preferences/Xhooks/edu.utah.scl.webcam.plist";

my $debug = ( defined $ARGV[0] ) ? 1 : 0;

# Check to see if I even have a built-in iSight
if ( `system_profiler SPUSBDataType | grep iSight | wc -l` < 1 ) {
  kill_self();
}

# Check to see if I should run on this computer
my $dict = loadDefaults( $prefFile );
my @ip_array = perlArrayFromNSArray( getPlistObject( $dict, "ip_list" ), 1 );
my $my_ip = get_ip();
my $run = 0;
foreach my $current_ip ( @ip_array ) {
  if ( compare_ips( $current_ip, $my_ip ) ) {
    $run = 1;
    last;
  }
}
if ( ! $run ) {
  kill_self();
}

# Read prefs
my $upload_server = readPrefFromCocoaDict( $dict, "upload_server", "example.com/cgi-bin/webcam_upload.cgi" );
my $upload_username = readPrefFromCocoaDict( $dict, "upload_username", "user" );
my $upload_password = readPrefFromCocoaDict( $dict, "upload_password", "pass" );
my $wait_seconds = readPrefFromCocoaDict( $dict, "wait_seconds", "300" );
my $image_save_path = readPrefFromCocoaDict( $dict, "image_save_path", "/tmp/$my_ip.png" );

while (1) {
  my $date = `date`;
  if ( ! $debug ) {
    unlink $image_save_path if -e $image_save_path;
    system "$isightcapture", $image_save_path;
    if ( -e $image_save_path ) {
      system "$AddTextToImage", "-i", $image_save_path, "-o", $image_save_path, "-s12", "-x9", "-y9", "-r0", "-g0", "-b0", "-d1", "$date";
      system "$AddTextToImage", "-i", $image_save_path, "-o", $image_save_path, "-s12", "-x9", "-y11`", "-r0", "-g0", "-b0", "-d1", "$date";
      system "$AddTextToImage", "-i", $image_save_path, "-o", $image_save_path, "-s12", "-x11", "-y9", "-r0", "-g0", "-b0", "-d1", "$date";
      system "$AddTextToImage", "-i", $image_save_path, "-o", $image_save_path, "-s12", "-x11", "-y11", "-r0", "-g0", "-b0", "-d1", "$date";
      system "$AddTextToImage", "-i", $image_save_path, "-o", $image_save_path, "-s12", "-x10", "-y10", "-d10", "$date";
    } else {
      print "Couldn't create image file!\n";
    }
  } else {
    system "echo debug > \"$image_save_path\"";
  }
  if ( -e $image_save_path ) {
    print "curl -u $upload_username:$upload_password -F file=\@\"$image_save_path\" \"$upload_server\" 2> /dev/null\n" if $debug;
    my $result = `curl -u $upload_username:$upload_password -F file=\@\"$image_save_path\" \"$upload_server\" 2> /dev/null`;
    print $result if $debug;
    unlink $image_save_path;
  }
  sleep $wait_seconds;
}

sub kill_self {
  # I shouldn't run, unload myself
  system "launchctl unload /Library/LaunchAgents/edu.utah.scl.webcam.plist";
  exit; # wont get here, and if it does, well launchd will run it again, so no point.  But who knows, stranger things have happened.
}

sub readPrefFromCocoaDict {
  my ( $dict, $key, $default_value ) = @_;
  my $bla = getPlistObject( $dict, $key );
  if ( ref $bla ne "SCALAR" ) {
    return perlValue ( $bla );
  } else {
    return $default_value;
  }
}
