#!/usr/bin/perl

# Config file contains IP info like so:
# old_ip new_ip subnet_mask router dns1 [ dns2...]

my $ncutil = "/usr/sbin/ncutil";
my $ip_change_file = $ARGV[0]; # Something like "/etc/entman/conf/changeiplist";
my $debug = 0;
my $print_usage_and_exit = 0;

if ( ! defined $ip_change_file ) {
  $print_usage_and_exit = 1;
}

if ( ! -f $ip_change_file ) {
  print "Can not find $ip_change_file\n\n";
  $print_usage_and_exit = 1;
}

if ( $print_usage_and_exit ) {
  print "Usage: $0 ip_change_list [ debug ]\n\n";
  print "The ip_change_list is a file that contains lines in this format:\n";
  print "old_ip  new_ip  subnet_mask   router  dns1    [ dns2 ...]\n\n";
  print "Here is an example:\n";
  print "1.2.3.4 4.3.2.1 255.255.255.0 4.3.2.1 4.3.1.1 4.3.2.2 4.3.3.3\n";
  print "1.2.3.5 4.3.2.2 255.255.255.0 4.3.2.1 4.3.1.1 4.3.2.2 4.3.3.3\n";
  print "1.2.3.6 4.3.2.3 255.255.255.0 4.3.2.1 4.3.1.1 4.3.2.2 4.3.3.3\n\n";
  print "If the machine's ip running on this box was \"1.2.3.4\", then\n";
  print "everything would be changed according to that line.  If the\n";
  print "machine's IP were \"1.2.3.5, everything would be changed according\n";
  print "to that line, etc.\"\n";
  exit;
}

if ( defined $ARGV[1] ) {
  print "Debugging only.  No changes will be made.\n";
  $debug = 1
}

$my_ip = get_ip();
if ( $my_ip eq "0.0.0.0" ) {
  print "I can't get my IP!\n";
  exit;  
}

my $my_line = `grep $my_ip \"$ip_change_file\"`;

$match = 0;
if ( $my_line ne "" ) {
  @lineparts = split ( ' ', $my_line );
  ( $old_ip, $new_ip, $subnet_mask, $router, @dns_server_list ) = @lineparts;
  if ( $old_ip eq $my_ip ) {
    $match = 1;
    print "Match found for $my_ip\n";
    print "$my_line\n";
    if ( $debug ) {
      # Set IP and other stuff
      print "\"$ncutil\" setprop \"/Automatic/Built-in Ethernet/IPv4\" ip-address \"$new_ip\"\n" if defined $new_ip;
      print "\"$ncutil\" setprop \"/Automatic/Built-in Ethernet/IPv4\" subnet-mask \"$subnet_mask\"\n" if defined $subnet_mask;
      print "\"$ncutil\" setprop \"/Automatic/Built-in Ethernet/IPv4\" router \"$router\"\n" if defined $router;
      foreach $dns_server (@dns_server_list) {
        print "\"$ncutil\" setprop -activate \"/Automatic/Built-in Ethernet/DNS\" name-server $dns_server\n";
      }
      # Turn it on
      print "\"$ncutil\" setprop -activate \"/Automatic/Built-in Ethernet/IPv4\" method Manual\n";
    } else {
      system "\"$ncutil\" setprop \"/Automatic/Built-in Ethernet/IPv4\" ip-address \"$new_ip\"" if defined $new_ip;
      system "\"$ncutil\" setprop \"/Automatic/Built-in Ethernet/IPv4\" subnet-mask \"$subnet_mask\"" if defined $subnet_mask;
      system "\"$ncutil\" setprop \"/Automatic/Built-in Ethernet/IPv4\" router \"$router\"" if defined $router;
      # Set DNS
      foreach $dns_server (@dns_server_list) {
        system "\"$ncutil\" setprop -activate \"/Automatic/Built-in Ethernet/DNS\" name-server $dns_server";
      }
      # Turn it on
      system "\"$ncutil\" setprop -activate \"/Automatic/Built-in Ethernet/IPv4\" method Manual";
    }
  }
}
if ( $match == 0 ) {
  print "No match found for $my_ip\n";
}

sub get_ip {
  ################################################
  # get_ip v1.c
  ################################################

  my $my_ip = `/sbin/ifconfig en0 | grep "inet " | cut -f2 -d' '`;
  chomp $my_ip;
  if ( $my_ip ne "" ) {
     return $my_ip;
  }
  return "0.0.0.0";
}
