#!/bin/bash
# Watch the progress of the current script
# wmertens@cisco.com

# Call this script with: path_to_me.sh ( -f | -k ) triggerfile output
# "triggerfile" is a file that is deleted when the script (that this script watches) runs. 
# "triggerfile" is created when the watched script finishes.
# "output" is the path of the watched script output.  STDERR should begin with "ERROR: "

export PATH=/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin

entman_config="/Library/Xhooks/Modules/xhooks/lib/config.sh"
entman_lib="/Library/Xhooks/Modules/xhooks/lib/common.sh"

if [ ! -f "${entman_config}" ]; then
	logger -s -t \"entman-`basename $0`\" -p user.info ${entman_config} is missing. Exiting...
	exit
fi

. "${entman_config}"
. "${entman_lib}"

#
# vars this script requires
#
ihook="${ENTMAN_IHOOK_PATH}"

if [ "$1" = "watch" ]; then
		
	script_is_running="${3}" # This script checks a particular PID and returns 1 if the PID is running.
	logfile="${4}" # this is where the log of the script is stored.
	ihook_window_title="${5}"
	ihook_window_size="${6}"
	
	if [ "$2" = "-f" ]; then
		# 'foreground' mode
		echo %WINDOWLEVEL HIGH
		echo %BECOMEKEY
		echo %UIMODE AUTOCRATIC
	elif [ "$2" = "-k" ]; then
		# 'key' mode
		echo %BECOMEKEY
	fi

	[ "${ihook_window_title}" != "" ] && echo %TITLE ${window_title}
	[ "${ihook_window_size}" != "" ] && echo %WINDOWSIZE ${ihook_window_size}

	count=30
	while [ $count -gt 0 ] && ! ${script_is_running}; do
		echo "Waiting $count seconds until script starts..."
		sleep 1
		count=$(( $count - 1 ))
	done
	if [ $count -le 0 ]; then
		echo "Script didn't start, bailing out!"
		sleep 2
		exit 0
	fi

	# Combined cat/tail -f/stderr-demux
	perl -e '
		$|=1;
		for (;;) {
			while (<>) {
				/^EXITCODE: / && next;
				if ( s/^ERROR: // ) {
					print STDERR;
				} else {
					print;
				}
			}
			sleep 1;
			# Reset the end-of-file indicator
			seek(STDIN,0,1);
		}' < $logfile &
	TAILPID=$!

	while ${script_is_running} ; do
		sleep 3
	done

	kill $TAILPID
	err=`tail -n 3 $logfile | awk '$1=="EXITCODE:"{print $2}'`
	err=$(( $err ))

	exit $err

else
	PROG="$0"
	case "$PROG" in /*) : ;; *) PROG="$PWD/$PROG" ;; esac
	${ihook} --no-titlebar --script="$PROG" watch "$@"
	exit 0
fi
