#!/usr/bin/perl

################################################################################
# Copyright (c) 2008 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################
# 
# This script is a general purpose perl emailing script (used for for emailing 
# reports in the University of Utah labs).
#
# It can email either the contents of a file or the file itself as an attachment.
#
# I used some information from Dave Wotton's emailer.sh script to do the MIME
# encoding.
#
################################################################################

use Getopt::Std;
use MIME::Base64 qw(encode_base64);
use File::Basename;

######
# Options
# t - to <parameter> (required)
# f - from <parameter>
# s - subject <parameter>
# m - message body <parameter>
# p - path to file <parameter> (required)(send as attachment default)
# a - uses path but send the file as text in the message body

# Get options and make sure t and p were specified
getopts("t:f:s:m:p:a",\%options);

my $email_recipient;
if( defined $options{'t'} ) {
	$email_recipient = $options{'t'};
} else {
	die "ERROR $0 - I need a email recipient";
}

my $path_to_file;
if( defined $options{'p'} ) {
	$path_to_file = $options{'p'};
} else {
	die "ERROR $0 - I need a path to a file to email";
}

# Check to make sure file specified exists
if(! -f $path_to_file) {
	die "ERROR $0 - this file does not exists: $path_to_file";
}

# Check for subject(s) and from(f) if they aren't declared use a default
my $subject;
if( defined $options{'s'} ) {
	$subject = $options{'s'};
} else {
	$subject = "";
}

my $from;
if( defined $options{'f'} ) {
	$from = $options{'f'};
} else {
	$from = "";
}

my @message_body = ();

if( defined $options{'m'} ) {
	push( @message_body, $options{'m'} );
	push( @message_body, "\n\n" );
}

if( ! defined $options{'a'} ) {
	# Open file handle to file path (p)
	if( ! open FILE_TO_EMAIL_HANDLE, "<", $path_to_file) {
		die "ERROR $0 - I could not open the file I am supposed to email"; 
	}
	
	while(<FILE_TO_EMAIL_HANDLE>) {
	 push(@message_body, $_);
	}
	
	close FILE_TO_EMAIL_HANDLE;
	
	open(SENDMAIL, "|/usr/sbin/sendmail -oi -t -odq") or die "Can't fork for sendmail: $!\n";
	print SENDMAIL << "THE EOF";
From: $from
To: $email_recipient
Subject: $subject
@message_body

THE EOF
	
	close(SENDMAIL) or warn "sendmail didn't close nicely!";
} else {
	if( ! open TEST, "<", $path_to_file) {
		die "ERROR $0 - I could not open the file I am supposed to email"; 
	}
	
	my $attachment;
	my $filename = basename($path_to_file);
	
	local($/) = undef;
	$attachment = encode_base64(<TEST>);
		
	open(SENDMAIL, "|/usr/sbin/sendmail -oi -t -odq") or die "Can't fork for sendmail: $!\n";
	print SENDMAIL << "THE_EOF";
From: $from
Subject: $subject
To: $email_recipient
Mime-Version: 1.0
Content-Type: multipart/mixed; boundary="DMW.Boundary.605592468"

This is a Mime message, which your mail program may not understand. Parts
of the message will appear as text. If the remainder appears as random
characters in the message body, instead of as attachments, then you'll
have to extract these parts and decode them manually.

--DMW.Boundary.605592468
Content-Type: text/plain; name="message.txt"; charset=US-ASCII
Content-Disposition: inline; filename="message.txt"
Content-Transfer-Encoding: 7bit
@message_body

--DMW.Boundary.605592468
Content-Type: application/octet-stream; name="$filename"
Content-Disposition: attachment; filename="$filename"
Content-Transfer-Encoding: base64

$attachment
--DMW.Boundary.605592468--
THE_EOF

	close(SENDMAIL) or warn "sendmail didn't close nicely!";
}

